##### The Red Book (v0.704442)
##### Scripts for Chapter 1
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")

getBasicPlot = function(x,y) {

  par( mar=c(3,3,0,0)+0.5 )
  par(family="serif", las=1)
  par(xaxs="i", yaxs="i")
  par(cex=0.6)
  par(font.lab=2, cex.lab=1.2)
  par(lend=1)

  plot.new()
  plot.window(xlim=c(0,10), ylim=c(0,10))

  axis(1, at=0:10); axis(2, at=0:10)
  title(xlab="x (Independent Variable)", line=2.25)
  title(ylab="y (Dependent Variable)", line=2.5)

}






##### ##### ##### ##### #####
### Figure 1.1 (p4)

set.seed(373)

pointColor=rgb(0.9,0.0,1.0)
lineColorOutside = "black"
lineColorInside = "pink"

x = seq(1,9)
n = length(x)
e = rnorm(n, m=0, s=3)
y = 2 + 0.5*x + e

getBasicPlot(x,y)

mod = lm(y~x)
xx=seq(min(x)-0.5,max(x)+0.5, length=2*n+1)
yy = predict(mod,newdata=data.frame(x=xx))


for(j in 1:9) {
  segments(j,yy[2*j],j,y[j], lty=3)
  err = bquote(epsilon[.(j)])
  text(j, 0.5*(yy[2*j]+y[j])-0.15, label=err, pos=4, cex=1.2)
}

lines(xx,yy, col=lineColorOutside, lwd=3)
lines(xx,yy, col=lineColorInside, lwd=1)

points(x,y, pch=21, bg=pointColor)




##### ##### ##### ##### #####
### Example 1.1
rm(list=ls())

x = c(-2,0,0,2)
y = c(3,0,2,-1)

mod = lm(y~x)
mean(x)
mean(y)

summary(mod)




##### ##### ##### ##### #####
### Figure 1.2
#   uses Example 1.1

par(mar=c(3,3,0,0)+0.5)
par(las=1, family="serif")
par(xaxs="i", yaxs="i")
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(-3,3), ylim=c(-4,4))

axis(1); axis(2)
title(xlab="X", line=2.5)
title(ylab="Y", line=2.25)
abline(mod)

points(x,y, pch=21, bg="dodgerblue")




##### ##### ##### ##### #####
### Example 1.2
rm(list=ls())

x = c("Male","Female","Female","Male")
y = c(3,1,2,2)

mod = lm(y~x)
summary(mod)




##### ##### ##### ##### #####
### Example 1.3
#   extends Example 1.2
rm(list=ls())

x = c(1,2,2,1)
y = c(3,1,2,2)

mod = lm(y~x)
summary(mod)

xbar = mean(x)
ybar = mean(y)

as.numeric( predict( mod, newdata=data.frame(x=mean(x)) ) )
ybar




##### ##### ##### ##### #####
### Example 1.4
#   extends Example 1.1
rm(list=ls())

x = c(-2,0,0,2)
y = c(3,0,2,-1)

Y = matrix(y)
X = cbind( c(1,1,1,1),x)

b = solve(t(X)%*%X) %*% t(X)%*%Y
b




##### ##### ##### ##### #####
### Figure 1.3
rm(list=ls())

par(mar=c(0,0,0,0), family="serif")

plot.new()
plot.window( xlim=c(0,1), ylim=c(0,1) )

# base plane
polygon( x=c(0,1,0.85,0.2), y=c(0,0,0.5,0.5) )

# triangle
o = c(0.25,0.25)
Y = c(0.75,1.0)
Yh = c(Y[1],0.4)

polygon( x=c(o[1],Yh[1],Y[1]), y=c(o[2],Yh[2],Y[2]), col=rgb(0.95,0.80,0.95,0.75) )

text(x=0.5,y=0.27,label=expression(hat(Y)==HY))
text(x=0.75,y=0.6,label=expression(E==(I-H)*Y),pos=4)
text(x=0.5,y=0.66,label=expression(Y))
text(o[1]-0.02,o[2]-0.02,label="O")

# rightangle
polygon( x=c(0.7111,0.75,0.75,0.71), y=c(0.388,0.40,0.4603,0.45), col="white")

# theta angle
center = o
radius = 0.14
theta  = seq(0.33, 1.74, length = 200)

xs = c(o[2],center[1]+theta/15 + radius * cos(theta) )
ys = c(o[2],center[2] + radius * sin(theta) )

polygon( x=xs, y=ys, col="white")
text(x=0.33,y=0.31,label=expression(theta))

# arrows
arrows( o[1],o[2],Y[1],Y[2], length=0.1)
arrows( o[1],o[2],Yh[1],Yh[2], length=0.1)
arrows( Yh[1],Yh[2],Y[1],Y[2], length=0.1)

points(o[1],o[2], pch=20)
points(Y[1],Y[2], pch=20)
points(Yh[1],Yh[2], pch=20)




##### ##### ##### ##### #####
### Example 1.5
rm(list=ls())

source("http://rfs.kvasaheim.com/rfs.R")

x = factor( c( rep("Wheat",4), rep("Corn",4), rep("Soybean",4) ) )
y = c( 722,965,940,756, 763,765,565,621, 566,658,540,485 )

meansMod = lm(y~0+x)
summary(meansMod)

xWheat = set.base(x, base="Wheat")
effectsMod = lm(y~xWheat)
summary(effectsMod)

xCorn = set.base(x, base="Corn")
effectsMod = lm(y~xCorn)
summary(effectsMod)




#####
##### ##### ##### ##### #####
#####
