##### The Red Book (v0.704442)
##### Scripts for Chapter 10
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")

library(lawstat)



##### ##### ##### ##### #####
### Section 10.3 (p317)

canada = read.csv("http://rur.kvasaheim.com/data/ocanada.csv")
summary(canada)
attach(canada)

## Models
depVar = cbind(correct,classSize-correct)

modL = glm( depVar ~ course+averageAge, family=binomial(link=make.link("logit")) )
modP = glm( depVar ~ course+averageAge, family=binomial(link=make.link("probit")) )
modC = glm( depVar ~ course+averageAge, family=binomial(link=make.link("cauchit")) )

# Assumption check
qchisq(c(0.025,0.975), df=15)

modL$deviance
modP$deviance
modC$deviance


## Quasilikelihood Model
QmodL = glm( depVar ~ course + averageAge, family=quasibinomial )
summary(QmodL)

# Model selection
QmodLreduced = glm( depVar ~ averageAge, family=quasibinomial )
anova(QmodLreduced,QmodL, test="LRT")

## Check assumptions
eL = residuals(modL)
runs.test(eL, order=averageAge)
summary(aov(eL~course))




### Figure 10.2 (p325)

pCorrect = correct/classSize


# Graphic
par(family="serif", las=1)
par(mar=c(4,4,0,0)+0.75)
par(xaxs="i", yaxs="i")
par(cex.lab=1.2,font.lab=2)

plot.new()
plot.window( xlim=c(17,22), ylim=c(0,1))

axis(1); axis(2)

title(xlab="Average Age", line=2.5)
title(ylab="Proportion Finding Canada", line=3)


# add curves
xx = seq(18,22, length=1e4)

yP = predict(modL, newdata=data.frame(averageAge=xx, course="Physical Science"))
lines(xx,logit.inv(yP), col="blue", lwd=2)

yS = predict(modL, newdata=data.frame(averageAge=xx, course="Social Science"))
lines(xx,logit.inv(yS), col="red", lwd=2)

yH = predict(modL, newdata=data.frame(averageAge=xx, course="Humanities"))
lines(xx,logit.inv(yH), col="gold", lwd=2)

yA = predict(modL, newdata=data.frame(averageAge=xx, course="Arts"))
lines(xx,logit.inv(yA), col="honeydew4", lwd=2)


# add points, properly colored
clip(0,90, -1,2)
points(averageAge[course=="Physical Science"],pCorrect[course=="Physical Science"], pch=21, bg="blue")
points(averageAge[course=="Social Science"],  pCorrect[course=="Social Science"],   pch=21, bg="red")
points(averageAge[course=="Humanities"],      pCorrect[course=="Humanities"],       pch=21, bg="gold")
points(averageAge[course=="Arts"],            pCorrect[course=="Arts"],             pch=21, bg="honeydew4")

text(17.9, 0.23, pos=2, label="Social Sciences")
text(17.9, 0.39, pos=2, label="Humanities")
text(17.9, 0.43, pos=2, label="Arts")
text(17.9, 0.71, pos=2, label="Physical Science")



detach(canada)
rm(list=ls())




##### ##### ##### ##### #####
### Section 10.4 (p327)
#   Sri Lanka Election

## Preamble
source("http://rfs.kvasaheim.com/rfs.R")
library(lawstat)

sri = read.csv("http://www.electoralforensics.org/datasets/sri2010pres.csv")
summary(sri)


## Clean the data
drop = which(sri$DIVISION=="Postal" | sri$DIVISION=="Displace")
sri2= sri[-drop, ]

attach(sri2)


## Models
pInv = REJECTED/TOTALVOTE
pRaj = RAJAPAKSE/(TOTALVOTE-REJECTED)
depVar = cbind(REJECTED,TOTALVOTE-REJECTED)

modL = glm( depVar~pRaj, family=binomial(link=make.link("logit")) )
summary(modL)

modP = glm( depVar~pRaj, family=binomial(link=make.link("probit")) )
summary(modP)

modC = glm( depVar~pRaj, family=binomial(link=make.link("cauchit")) )
summary(modC)

modLL = glm( depVar~pRaj, family=binomial(link=make.link("loglog")) )
summary(modLL)

modCL = glm( depVar~pRaj, family=binomial(link=make.link("cloglog")) )
summary(modCL)



qchisq(c(0.025,0.975), df=184)


QmodL = glm( depVar~pRaj, family=quasibinomial(link=make.link("logit")) )
eL = residuals(QmodL)

runs.test(eL, order=pRaj)

summary(QmodL)



### Figure 10.4 (p330)

par(family="serif", las=1)
par(mar=c(4,4,0,0)+0.75)
par(xaxs="i", yaxs="i")
par(cex.lab=1.2,font.lab=2)

plot.new()
plot.window( xlim=c(0,1), ylim=c(0,0.06))

axis(1); axis(2)

title(xlab="Support for Mahinda Rajapaksa", line=2.5)
title(ylab="Invalidation Rate", line=3)

# Create lines
xx = seq(0,1, length=1e4)

yL = predict(modL, newdata=data.frame(pRaj=xx))
lines(xx,logit.inv(yL), col="blue", lwd=2)

yP = predict(modP, newdata=data.frame(pRaj=xx))
lines(xx,probit.inv(yP), col="red", lwd=2)

yC = predict(modC, newdata=data.frame(pRaj=xx))
lines(xx,cauchit.inv(yC), col="gold4", lwd=2)

yLL = predict(modLL, newdata=data.frame(pRaj=xx))
lines(xx,loglog.inv(yLL), col="brown", lwd=2)

yCL = predict(modCL, newdata=data.frame(pRaj=xx))
lines(xx,cloglog.inv(yCL), col="brown", lwd=2)

legend("topright",bty="n",lwd=2,
	col=c("blue","red","gold4","green4","brown"),
	legend=c("Logit","Probit","Cauchit","Log-log","Complementary log-log") 
	)

clip(0,90, -1,2)
points(pRaj,pInv, pch=21, bg="pink")





### Figure 10.5 (p331)
#   Graphic without dropping Postal and Displaced votes

detach(sri2)
attach(sri)

pInv = REJECTED/TOTALVOTE
pRaj = RAJAPAKSE/(TOTALVOTE-REJECTED)
depVar = cbind(REJECTED,TOTALVOTE-REJECTED)

# Models
modL  = glm( depVar~pRaj, family=binomial(link=make.link("logit")) )
modP  = glm( depVar~pRaj, family=binomial(link=make.link("probit")) )
modC  = glm( depVar~pRaj, family=binomial(link=make.link("cauchit")) )
modLL = glm( depVar~pRaj, family=binomial(link=make.link("loglog")) )
modCL = glm( depVar~pRaj, family=binomial(link=make.link("cloglog")) )


# Graphic
par(family="serif", las=1)
par(mar=c(4,4,0,0)+0.75)
par(xaxs="i", yaxs="i")
par(cex.lab=1.2,font.lab=2)

plot.new()
plot.window( xlim=c(0,1), ylim=c(0,0.06))

axis(1); axis(2)

title(xlab="Support for Mahinda Rajapaksa", line=2.5)
title(ylab="Invalidation Rate", line=3)

# Create lines
xx = seq(0,1, length=1e4)

yL = predict(modL, newdata=data.frame(pRaj=xx))
lines(xx,logit.inv(yL), col="blue", lwd=2)

yP = predict(modP, newdata=data.frame(pRaj=xx))
lines(xx,probit.inv(yP), col="red", lwd=2)

yC = predict(modC, newdata=data.frame(pRaj=xx))
lines(xx,cauchit.inv(yC), col="gold4", lwd=2)

yLL = predict(modLL, newdata=data.frame(pRaj=xx))
lines(xx,loglog.inv(yLL), col="brown", lwd=2)

yCL = predict(modCL, newdata=data.frame(pRaj=xx))
lines(xx,cloglog.inv(yCL), col="brown", lwd=2)

legend("topright",bty="n",lwd=2,
	col=c("blue","red","gold4","green4","brown"),
	legend=c("Logit","Probit","Cauchit","Log-log","Complementary log-log") 
	)

clip(0,90, -1,2)
points(pRaj,pInv, pch=21, bg="pink")














#####
##### ##### ##### ##### #####
#####
