##### The Red Book (v0.704442)
##### Scripts for Chapter 11
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")

library(lawstat)



##### ##### ##### ##### #####
### Section 11.1 (p341)

## Create data
set.seed(577)

n = 75
x1 = sort( runif(n, min=0, max=2) )
beta0 = 0
beta1 = 2

lambda = exp( beta0 + beta1*x1 )
y = rpois(n, lambda)


# Models

mod1 = lm(y ~ x1)

depVar = log(y+1)
mod2 = lm(depVar ~ x1)

mod3 = glm(y ~ x1, family=poisson)



### Figure 11.1

par(las=1, family="serif")
par(mar=c(4,4,0,0)+0.5)
par(xaxs="i",yaxs="i")
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0,2), ylim=c(0,60))

axis(1, at=seq(0,2,length=11))
axis(2)
title(xlab="X", line=2.5)
title(ylab="Y", line=2.8)

xx = seq(0,2, length=1e4)
y1 = predict(mod1, newdata=data.frame(x1=xx))
y2 = predict(mod2, newdata=data.frame(x1=xx))
y3 = predict(mod3, newdata=data.frame(x1=xx))
yt = exp(beta0 + beta1*xx ) 

lines(xx, yt, col="black", lwd=2)
lines(xx, y1, col="red2", lwd=1)
lines(xx, exp(y2)-1, col="green4", lwd=1)
lines(xx, exp(y3), col="blue1", lwd=1)

clip(-1,100, -1,100)
points(x1, y, pch=21, bg="honeydew3")




##### ##### ##### ##### #####
### Example 11.1 (p347)

## Clean
rm(list=ls())

## Preamble
source("http://rfs.kvasaheim.com/rfs.R")
library(lawstat)

# Data
vcr = read.csv("http://rur.kvasaheim.com/data/crime.csv")

## Model
m2 = glm( inituse~pop90, family=poisson(link=log), data=vcr, subset=(ccode!=93) )


## Predict Utah
UTAH = data.frame(pop90=1722850)
prL = predict(m2, newdata=UTAH)
exp(prL)



### Figure 11.2 (p349)

newX <- 0:400*1e5
pr <- predict(m2, newdata=data.frame(pop90=newX))
pr <- exp(pr)

# Graphic
par(mar=c(5,4,1,1)+0.1)

plot(inituse~pop90, las=1, pch=16,xlab="State Population, 1990\n(millions)",
  ylab="Initiative Use", xaxt="n",xlim=c(0,2e7), data=vcr)
axis(1,labels=0:3*10,at=0:3*1e7)
lines(newX,pr, col=4, lwd=2)








##### ##### ##### ##### #####
### Section 11.3 (p350)
#   requires Example 11.1

source("http://rfs.kvasaheim.com/rfs.R")

# Dispersion factor
m2$deviance/m2$df.residual
sqrt( m2$deviance/m2$df.residual )

# Table 11.1
summary(m2)


# Table 11.2
m2q = glm(inituse~pop90, family=quasipoisson(link=log), data=vcr, subset=(ccode!=93))
summary(m2q)



# Table 11.3
library(MASS)

m2n = glm.nb(inituse~pop90, data=vcr, subset=(ccode!=93))
summary(m2n)


prN = predict(m2n, newdata=UTAH)
exp(prN)









##### ##### ##### ##### #####
### Section 11.4 (p355)
#   Example 11.2: Body Counts

## Clean
rm(list=ls())

## Preamble
library(MASS)

terrorism = read.csv("http://rur.kvasaheim.com/data/terrorism.csv")
names(terrorism)
attach(terrorism)


### Table 11.4
# Option One: as an independent variables
model.1p <- glm(pira ~ riteleft + days, poisson)
model.1q <- glm(pira ~ riteleft + days, quasipoisson)
model.1n <- glm.nb(pira ~ riteleft + days)
summary(model.1p)
summary(model.1q)
summary(model.1n)


### Table 11.5
# Option Two: as an offset
model.2p <- glm(pira ~ riteleft, offset=log(days), poisson)
model.2q <- glm(pira ~ riteleft, offset=log(days), quasipoisson)
model.2n <- glm.nb(pira ~ riteleft, offset(log(days)), data=terrorism )
summary(model.2p)
summary(model.2q)
summary(model.2n)



### Section 11.4.1
#   Bettering the fit

formula = "pira ~ riteleft + labour + I(riteleft^2) + I(riteleft^3) + I(riteleft^4) + I(labour*riteleft)"
model.3q <- glm(formula, offset=log(days), quasipoisson)
model.3n <- glm.nb(formula, offset(log(days)), data=terrorism)
summary(model.3q)
summary(model.3n)


## Reduce model

formula = "pira ~ riteleft + labour + I(riteleft^2) + I(riteleft^3) + I(riteleft^4) + labour"

## Table 11.6
model.3q <- glm(formula, offset=log(days), quasipoisson)
model.3n <- glm.nb(formula, offset(log(days)), data=terrorism)
summary(model.3q)
summary(model.3n)

# Yay! Statistical significance!



### Figure 11.3 (p361)

x <- -50:50
prediction.q0 <- exp(predict(model.3q, newdata=data.frame(riteleft=x, labour=0, days=365) ))
prediction.q1 <- exp(predict(model.3q, newdata=data.frame(riteleft=x, labour=1, days=365) ))
prediction.n0 <- exp(predict(model.3n, newdata=data.frame(riteleft=x, labour=0, days=365) ))
prediction.n1 <- exp(predict(model.3n, newdata=data.frame(riteleft=x, labour=1, days=365) ))

# Plot Left: quasi-Poisson
plot(riteleft, total/days*365, type="n", xlim=c(-75,75), col=2, las=1, 
	ylab="Annualized Deaths due to Terrorism", 
	xlab="Level of conservatism in the UK Parliament" )
abline(v=0, col='grey')
points(riteleft[labour==1], total[labour==1]/days[labour==1]*365, col=4, pch=16) 
points(riteleft[labour==0], total[labour==0]/days[labour==0]*365, col=2, pch=17) 
lines(x,prediction.q0, col=4, lwd=2)
lines(x,prediction.q1, col=2, lwd=2)

# Plot right: negative binomial
plot(riteleft, total/days*365, type="n", xlim=c(-75,75), col=2, las=1, 
	ylab="Annualized Deaths due to Terrorism", 
	xlab="Level of conservatism in the UK Parliament" )
abline(v=0, col='grey')
points(riteleft[labour==1], total[labour==1]/days[labour==1]*365, col=4, pch=16) 
points(riteleft[labour==0], total[labour==0]/days[labour==0]*365, col=2, pch=17) 
lines(x,prediction.n0, col=4, lwd=2)
lines(x,prediction.n1, col=2, lwd=2)







#####
##### ##### ##### ##### #####
#####
