##### The Red Book (v0.704442)
##### Scripts for Chapter 12
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")

library(lawstat)



##### ##### ##### ##### #####
### Section 12.1 (p377)
#   Nominal Dependent Variable


dt = read.csv("http://rur.kvasaheim.com/data/gssocc.csv")
attach(dt)


## Sample statistics
summary(white)

median(ed)
mean(ed)
hildebrand.rule(ed)
quartile(ed)
mean(ed<=12)
mean(ed>=16)
mean(ed<12)

summary(exper)
hildebrand.rule(exper)


cor( cbind(white,ed,exper) )
cor.test(ed,exper)

BOB = data.frame( white=1, ed=20, exper=10 )


## The Blue Collar model
bluecol = (occ=="BlueCol")
modBlue = glm( bluecol ~ white + ed + exper, family=binomial)

## The White Collar model
whitecol = (occ=="WhiteCol")
modWhite = glm( whitecol ~ white + ed + exper, family=binomial)

## The Menial model
menial = (occ=="Menial")
modMenial = glm( menial ~ white + ed + exper, family=binomial)

## The Craft model
craft = (occ=="Craft")
modCraft = glm( craft ~ white + ed + exper, family=binomial)

## The Professional model
professional = (occ=="Prof")
modProf = glm( professional ~ white + ed + exper, family=binomial)


### Table 12.2
summary(modBlue)


### The predictions for Bob
logit.inv( predict(modBlue, newdata=BOB) )
logit.inv( predict(modWhite, newdata=BOB) )
logit.inv( predict(modMenial, newdata=BOB) )
logit.inv( predict(modCraft, newdata=BOB) )
logit.inv( predict(modProf, newdata=BOB) )

# Total probability predicted:
logit.inv( predict(modBlue, newdata=BOB) )   +
logit.inv( predict(modWhite, newdata=BOB) )  +
logit.inv( predict(modMenial, newdata=BOB) ) +
logit.inv( predict(modCraft, newdata=BOB) )  +
logit.inv( predict(modProf, newdata=BOB) )



### Multinomial modeling

library(nnet)

# Null Model
model.null = multinom(occ ~ 1)

# Research Model
model.mn1 = multinom(occ ~ white + ed + exper)

# Predict Bob
predict(model.mn1, newdata=BOB, type="probs")

# Sums to unity!
sum(predict(model.mn1, newdata=BOB, type="probs"))


### Table 12.3
summary(model.mn1)


### Goodness of fit

# Relative accuracy
table(occ)
table( predict(model.mn1), occ )

((37+43+0+89+0)/337) / (140/337)

# AIC
AIC(model.mn1)
AIC(model.null)













##### ##### ##### ##### #####
### Section 12.2 (p383)
#   Ordinal Dependent Variables

## Clean
detach(dt)
rm(list=ls())



### Figure 12.1 (p384)

# data
x <- 0:30
y <- 0.8*x+1

plot(x,y, type="l", xlim=c(0,30), ylim=c(0,30), lwd=2, las=1, yaxt="n", ylab=expression(eta) )

abline(h=5)
abline(h=15)
abline(h=20)

t1 = expression(tau[1])
t2 = expression(tau[2])
t3 = expression(tau[3])
axis(2, label=c(t1, t2, t3), at=c(5, 15, 20), las=1)
axis(2, label=c("A", "B", "C", "D"), at=c(2, 10, 17.5, 25), las=1)




### Example 12.3 (p385)

## Preamble
library(MASS)

dt = read.csv("http://rur.kvasaheim.com/data/ordwarm.csv")
summary(dt)

# Ensure the warmth is seen as ordinal
dt$warm = ordered(dt$warm, levels=c("SD","D","A","SA"))

attach(dt)


## Model
model.ol1 = polr( warm ~ male + white + age + ed )

# Predict Bob
BOB = data.frame(male="Men", white="White", age=40, ed=20)
predict(model.ol1, newdata=BOB, type="probs")


## Accuracy

# Null Accuracy
table(warm) 

# Model Accuracy
table(warm,predict(model.ol1))
sum( diag( table(warm,predict(model.ol1)) ) ) # Number Correct

# Relative accuracy 
946/856













##### ##### ##### ##### #####
### Section 12.3 (p388)
#   Cattle Feed (Nominal)

## Clean
detach(dt)
rm(list=ls())

## Preamble
library(nnet)
cowz = read.csv("http://rur.kvasaheim.com/data/cattleData.csv")
summary(cowz)
attach(cowz)

mysteryMoo = data.frame(weight=1197, age=21, ranch="RUR")

# Information
cor.test(weight,age)
table(ranch,feedType)


## Model
cowModel = multinom(feedType ~ weight + ranch + age)
summary(cowModel)

# Predictions
predict(cowModel, mysteryMoo, type="prob")





### Figure 12.2 (p392)

# Get predictions
theWeights = seq(1019,1579,length=1e4)
theAge = median(age)
prRUR = predict( cowModel,newdata=data.frame(weight=theWeights, age=theAge, ranch="RUR"), type="probs" )

# The Graphic
par(family="serif", las=1)
par(xaxs="i", yaxs="i")
par(mar=c(4,4,0,0)+0.5)
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(1000,1600), ylim=c(0,1))

axis(1); axis(2)

title(xlab="Weight [lb]")
title(ylab="Probability at RUR Ranch")

lines(theWeights,prRUR[,1], col=1) # Accuration
lines(theWeights,prRUR[,2], col=2) # Purina
lines(theWeights,prRUR[,3], col=3) # Rangeland
lines(theWeights,prRUR[,4], col=4) # Steakmaker
lines(theWeights,prRUR[,5], col=5) # Wind and Rain

legend("topright", bty="n", col=1:5, lwd=2,
    legend=c("Accuration","Purina","Rangeland","Steakmaker","Windand Rain")
    )














##### ##### ##### ##### #####
### Section 12.4 (p393)
#   SUvR (Ordinal)

## Clean
detach(cowz)
rm(list=ls())

## Preamble
library(MASS)

suvrData = read.csv("http://rur.kvasaheim.com/data/suvr.csv")
suvrData$class = ordered(suvrData$class, levels=c("Non-Matriculated", "Freshman", "Sophomore", "Junior", "Senior") )
summary(suvrData)

attach(suvrData)


# Information
table(gender, class)


## Null Model
suvrNullModel = polr(class ~ 1, data=suvrData)
summary(suvrNullModel)


## Model
suvrModel = polr(class~gender + gpa, data=suvrData)
summary(suvrModel)

# Prediction
eliska = data.frame(gender="Female", gpa=3.33)
predict(scuvrModel, eliska, type="prob")




### Figure 12.3

# Make the predictions
theGPAs = seq(0,4,length=1e4)
prMALE   = predict(suvrModel, newdata=data.frame(gpa=theGPAs, gender="Male"),   type="probs")
prFEMALE = predict(suvrModel, newdata=data.frame(gpa=theGPAs, gender="Female"), type="probs")


# Graphic
par(family="serif", las=1)
par(xaxs="i", yaxs="i")
par(mar=c(4,4,0,0)+0.5)
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0,4), ylim=c(0,1))

axis(1); axis(2)
title(xlab="GPA")
title(ylab="Level Probability")

# Male
lines(theGPAs,prMALE[,1], col=2) # Non-Matriculated
lines(theGPAs,prMALE[,2], col=3) # Freshman
lines(theGPAs,prMALE[,3], col=4) # Sophomore
lines(theGPAs,prMALE[,4], col=5) # Junior
lines(theGPAs,prMALE[,5], col=6) # Senior

# Female
lines(theGPAs,prFEMALE[,1], col=2, lty=2) # Non-Matriculated
lines(theGPAs,prFEMALE[,2], col=3, lty=2) # Freshman
lines(theGPAs,prFEMALE[,3], col=4, lty=2) # Sophomore
lines(theGPAs,prFEMALE[,4], col=5, lty=2) # Junior
lines(theGPAs,prFEMALE[,5], col=6, lty=2) # Senior

# Legend
legend("topright", bty="n", col=c(2:6,NA,1,1), lwd=2, lty=c(1,1,1,1,1,NA,1,2),
  legend=c("Non-Matriculated","Freshman","Sophomore","Junior","Senior", "", "Male","Female")
  )














#####
##### ##### ##### ##### #####
#####
