##### The Red Book (v0.704442)
##### Scripts for Chapter 2
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")

getBasicPlot = function(x,y) {

  par( mar=c(3,3,0,0)+0.5 )
  par(family="serif", las=1)
  par(xaxs="i", yaxs="i")
  par(cex=0.6)
  par(font.lab=2, cex.lab=1.2)
  par(lend=1)

  plot.new()
  plot.window(xlim=c(0,10), ylim=c(0,10))

  axis(1, at=0:10); axis(2, at=0:10)
  title(xlab="x (Independent Variable)", line=2.25)
  title(ylab="y (Dependent Variable)", line=2.5)

}






##### ##### ##### ##### #####
### Figure 2.1 (p48)

set.seed(373)

pointColor=rgb(0.9,0.0,1.0)
lineColorOutside = "black"
lineColorInside = "pink"

x = seq(1,9)
n = length(x)
e = rnorm(n, m=0, s=3)
y = 2 + 0.5*x + e

getBasicPlot(x,y)

mod = lm(y~x)
xx=seq(min(x)-0.5,max(x)+0.5, length=2*n+1)
yy = predict(mod,newdata=data.frame(x=xx))


for(j in 1:9) {
  segments(j,yy[2*j],j,y[j], lty=3)
  err = bquote(epsilon[.(j)])
  text(j, 0.5*(yy[2*j]+y[j])-0.15, label=err, pos=4, cex=1.2)
}

lines(xx,yy, col=lineColorOutside, lwd=3)
lines(xx,yy, col=lineColorInside, lwd=1)

points(x,y, pch=21, bg=pointColor)






##### ##### ##### ##### #####
### Figure 2.2 (p57)

getBasicPlot(x,y)

mod = lm(y~x)
xx=seq(min(x)-0.5,max(x)+0.5, length=2*n+1)
yy = predict(mod,newdata=data.frame(x=xx))
yy1= predict(mod,newdata=data.frame(x=xx), interval="confidence")
yy2= predict(mod,newdata=data.frame(x=xx), interval="prediction")

lines(xx,yy, col=lineColorOutside, lwd=1)

segments( 4.5, yy1[9,"lwr"], 4.5,yy1[9,"upr"], lwd=3, col="black")
segments( 4.5, yy2[9,"lwr"], 4.5,yy2[9,"upr"], lwd=1, col="grey60")
points(4.5,yy[9], pch=21, bg=pointColor, cex=1.5)





##### ##### ##### ##### #####
### Figure 2.3 (p62)
#   t-distribution confidence interval/rejection region

# parameters
alpha=0.05
nu = 9
xx = seq(-5,5, length=1e4)
yy = dt(xx, df=nu)

# figure
par( mar=c(6,0,0,0)+0.5 )
par(family="serif", las=1)
par(xaxs="i", yaxs="i")
par(cex=0.6)
par(font.lab=2, cex.lab=1.2)
par(lend=1)

plot.new()
plot.window(xlim=c(-5,5), ylim=c(0,max(yy)))

lcp = qt(alpha/2, df=nu)
xcp = seq(-10,lcp, length=1e4)
ycp = dt(xcp,df=nu)
polygon( x=c(xcp,rev(xcp)), y=c(ycp,rep(0,length(ycp))), col="dodgerblue")

ucp = qt(1-alpha/2, df=nu)
xcp = seq(ucp,20, length=1e4)
ycp = dt(xcp,df=nu)
polygon( x=c(xcp,rev(xcp)), y=c(ycp,rep(0,length(ycp))), col="dodgerblue")

lines(xx,yy)

axis(1, at=seq(-5,5,1))
axis(1, at=0, label="T", line=1.75, cex.axis=1.7, tick=FALSE)
axis(1, at=0, label=expression(bar(X)), line=4.5, cex.axis=1.5, tick=FALSE)
axis(1, at=2.262*c(1,-1), label=c("",""))
axis(1, at=-2.262, label=-2.262, line=1.75, cex.axis=1.5, tick=FALSE)
axis(1, at=+2.262, label=+2.262, line=1.75, cex.axis=1.5, tick=FALSE)

eqn = expression( bar(x) - 2.262 ~ s/sqrt(n) )
axis(1, at=-2.262, label=eqn, line=4.5, cex.axis=1.5, tick=FALSE)
eqn = expression( bar(x) + 2.262 ~ s/sqrt(n) )
axis(1, at=+2.262, label=eqn, line=4.5, cex.axis=1.5, tick=FALSE)

eqn = "\u2190"
axis(1, at=-0.75, line=1.75, label=eqn, tick=FALSE, cex.axis=2.5)
axis(1, at=-0.75, line=4.5, label=eqn, tick=FALSE, cex.axis=2.5)
eqn = "\u2192"
axis(1, at=0.75, line=1.75, label=eqn, tick=FALSE, cex.axis=2.5)
axis(1, at=0.75, line=4.5, label=eqn, tick=FALSE, cex.axis=2.5)


# Note: The arrows may not display. It depends on the fonts you have
# installed on your system. When using basic R, you will see the arrows.
# RStudio will most likely display empty boxes.




##### ##### ##### ##### #####
### Figure 2.4
#   illustration of chi-square central confidence interval/rejection region

# parameters
alpha=0.10
nu = 4
xx = seq(0,15, length=1e4)
yy = dchisq(xx, df=nu)

# figure
par( mar=c(3,0,0,0)+0.5 )
par(family="serif", las=1)
par(xaxs="i", yaxs="i")
par(cex=0.6)
par(font.lab=2, cex.lab=1.2)
par(lend=1)

plot.new()
plot.window(xlim=c(0,15), ylim=c(0,max(yy)))

lcp = qchisq(alpha/2, df=nu)
xcp = seq(0,lcp, length=1e4)
ycp = dchisq(xcp,df=nu)
polygon( x=c(xcp,rev(xcp)), y=c(ycp,rep(0,length(ycp))), col="dodgerblue")

ucp = qchisq(1-alpha/2, df=nu)
xcp = seq(ucp,20, length=1e4)
ycp = dchisq(xcp,df=nu)
polygon( x=c(xcp,rev(xcp)), y=c(ycp,rep(0,length(ycp))), col="dodgerblue")

lines(xx,yy)

axis(1, at=seq(0,16,2))
title(xlab="x", line=2.25)
axis(1, at=c(lcp,ucp), label=c("",""))

# Width:
ucp-lcp




##### ##### ##### ##### #####
### Figure 2.5
#   illustration of chi-square min-width confidence interval/rejection region

# parameters
lcp = 0.167
ucp = 7.881

dchisq(lcp,df=nu)
dchisq(ucp,df=nu)
pchisq(ucp,df=nu) - pchisq(lcp,df=nu)


# figure
par( mar=c(3,0,0,0)+0.5 )
par(family="serif", las=1)
par(xaxs="i", yaxs="i")
par(cex=0.6)
par(font.lab=2, cex.lab=1.2)
par(lend=1)

plot.new()
plot.window(xlim=c(0,15), ylim=c(0,max(yy)))

xcp = seq(0,lcp, length=1e4)
ycp = dchisq(xcp,df=nu)
polygon( x=c(xcp,rev(xcp)), y=c(ycp,rep(0,length(ycp))), col="dodgerblue")

xcp = seq(ucp,20, length=1e4)
ycp = dchisq(xcp,df=nu)
polygon( x=c(xcp,rev(xcp)), y=c(ycp,rep(0,length(ycp))), col="dodgerblue")

lines(xx,yy)

axis(1, at=seq(0,16,2))
title(xlab="x", line=2.25)
axis(1, at=c(lcp,ucp), label=c("",""))

# Width:
ucp-lcp

#  improvement
(8.777-7.714)/8.777






#####
##### ##### ##### ##### #####
#####
