##### The Red Book (v0.704442)
##### Scripts for Chapter 3
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")





##### ##### ##### ##### #####
### Section 3.1.1 (p73)

set.seed(370)

# Normal residuals
e = rnorm(100, m=0, s=2)

# skewed residuals
enon = rexp(100, rate=1/10)

# q-q plots
qqnorm(e)
qqline(e)

qqnorm(enon)
qqline(enon)

# histograms
hist(e)
normoverlay(e)
hist(enon)
normoverlay(enon)




##### ##### ##### ##### #####
### Section 3.1.2 (p78)

set.seed(370)

# Normal
e = rnorm(100, m=0, s=2)
head(e, 5)

shapiroTest(e)

# not Normal
set.seed(370)
e = rt(100, df=100)
shapiroTest(e)

# severaly not Normal
set.seed(370)
e = rexp(100, rate=1/10)
shapiroTest(e)




##### ##### ##### ##### #####
### Section 3.1.3 (p81)

# create data
set.seed(30)
beta0 = 3
beta1 = 0
x = 1:20
e = rnorm(20, m=0, s=1)
y = beta0 + beta1*x + e

# model data
model = lm(y~x)
summary(model)

summary(model)[[4]][2,4]



## Normal residuals
set.seed(30)

pval = numeric()
beta0 = 3
beta1 = 0
x = 1:20

for(i in 1:1e4) {
  e = rnorm(20, m=0, s=1)
  y = beta0 + beta1*x + e
  model = lm(y~x)
  pval[i] = summary(model)[[4]][2,4]
}

hist(pval)
ks.test(pval, "punif")
binom.test( x=sum(pval<0.05), n=length(pval), p=0.05 )



## Non-Normal Residuals I
set.seed(30)
pval = numeric()
beta0 = 3
beta1 = 0
x = 1:20
for(i in 1:1e4) {
  e = rexp(20, rate=1)
  y = beta0 + beta1*x + e
  model = lm(y~x)
  pval[i] = summary(model)[[4]][2,4]
}

hist(pval)
ks.test(pval, "punif")
binom.test( x=sum(pval<0.05), n=length(pval), p=0.05 )


## Non-Normal Residuals II
set.seed(30)
pval = numeric()
beta0 = 3
beta1 = 0
x = 1:20
for(i in 1:1e4) {
  e = rchisq(20, df=1)
  y = beta0 + beta1*x + e
  model = lm(y~x)
  pval[i] = summary(model)[[4]][2,4]
}

hist(pval)
ks.test(pval, "punif")
binom.test( x=sum(pval<0.05), n=length(pval), p=0.05 )



## Non-Normal Residuals III
# n=20
set.seed(30)
pval = numeric()
beta0 = 3
beta1 = 0
x = 1:20
for(i in 1:1e4) {
  e = rcauchy(20)
  y = beta0 + beta1*x + e
  model = lm(y~x)
  pval[i] = summary(model)[[4]][2,4]
}

hist(pval)
ks.test(pval, "punif")
binom.test( x=sum(pval<0.05), n=length(pval), p=0.05 )


# n=5000
set.seed(30)
pval = numeric()
beta0 = 3
beta1 = 0
x = 1:5000
for(i in 1:1e4) {
  e = rcauchy(5000)
  y = beta0 + beta1*x + e
  model = lm(y~x)
  pval[i] = summary(model)[[4]][2,4]
}

hist(pval)
ks.test(pval, "punif")
binom.test( x=sum(pval<0.05), n=length(pval), p=0.05 )




##### ##### ##### ##### #####
### Figure 3.4 (p89)
set.seed(370)
x = seq(0,3,length=20)
n = length(x)
e = rnorm(n)
y = 4 + 2*x^2 + e
mod = lm(y~x)
E = residuals(mod)
colores = c("dodgerblue","pink")[1+(E<0)]
plot(x,E, pch=21, bg=colores)



##### ##### ##### ##### #####
### Figure 3.5 (p90)
#   uses Figure 3.4
x2 = x^2
mod = lm(y~x+x2)
E2 = residuals(mod)

colores = c("dodgerblue","pink")[1+(E2<0)]
plot(x,E2, pch=21, bg=colores)
plot(x2,E2, pch=21, bg=colores)




##### ##### ##### ##### #####
### Section 3.2.2 (p91)
library(lawstat)

set.seed(370)
x = runif(100)
e = rnorm(100)
runs.test(e, order=x)




##### ##### ##### ##### #####
### Figure 3.6 (p95)


# homoskedasticity
set.seed(370)
x = seq(0,1000)
n = length(x)


## homoskedastic
e = rnorm(n)

par(cex=0.8)
par(xaxs="i", yaxs="i")
par(mar=c(1,3,4,2)+1)
par(family="serif", las=1)

plot.new()
plot.window( xlim=c(0,1000), ylim=c(-4,4))

axis(3, at=seq(0,1000,200)); axis(2, at=-5:5)
mtext(side=3, text="Independent Variable, x", line=2.5, cex=0.8, las=0)
title(ylab="Residuals, e", line=2.5)

abline(h=0, col="grey", lty=3)
clip(-1e3,1e5,-5,5)
points(x,e, pch=21, bg="dodgerblue")


## heteroskedastic - trumpet
e = rnorm(n,m=0,s=x/500)

par(cex=0.8)
par(xaxs="i", yaxs="i")
par(mar=c(1,2,4,3)+1)
par(family="serif", las=1)

plot.new()
plot.window( xlim=c(0,1000), ylim=c(-4,4))

axis(3, at=seq(0,1000,200)); axis(4, at=-5:5)
mtext(side=3, text="Independent Variable, x", line=2.5, cex=0.8, las=0)
mtext(side=4, text="Residuals, e", line=2.5, cex=0.8, las=0)

abline(h=0, col="grey", lty=3)
clip(-1e3,1e5,-5,5)
points(x,e, pch=21, bg="dodgerblue")


## heteroskedastic - funnel
e = rnorm(n,m=0,s=2-x/500)

par(cex=0.8)
par(xaxs="i", yaxs="i")
par(mar=c(3,3,1,2)+1)
par(family="serif", las=1)

plot.new()
plot.window( xlim=c(0,1000), ylim=c(-4,4))

axis(1, at=seq(0,1000,200)); axis(2, at=-5:5)
title(xlab="Independent Variable, x", line=2.5)
title(ylab="Residuals, e", line=2.5)

abline(h=0, col="grey", lty=3)
clip(-1e3,1e5,-5,5)
points(x,e, pch=21, bg="dodgerblue")


## heteroskedastic - bulge
e = rnorm(n,m=0,s=1.5-abs(1-x/500))

par(cex=0.8)
par(xaxs="i", yaxs="i")
par(mar=c(3,2,1,3)+1)
par(family="serif", las=1)

plot.new()
plot.window( xlim=c(0,1000), ylim=c(-4,4))

axis(1, at=seq(0,1000,200)); axis(4, at=-5:5)
title(xlab="Independent Variable, x", line=2.5)
mtext(side=4, text="Residuals, e", line=2.5, cex=0.8, las=0)

abline(h=0, col="grey", lty=3)
clip(-1e3,1e5,-5,5)
points(x,e, pch=21, bg="dodgerblue")




##### ##### ##### ##### #####
### Section 3.3.2 (p96)
library(lmtest)

# homoskedastic
set.seed(30)
x = seq(0,100)
n = length(x)
e = rnorm(n, m=0, s=1)
y = 3 + 2*x + e
plot(x,y, pch=21, bg="dodgerblue")

mod = lm(y~x)
bptest(mod)

# heteroskedastic
set.seed(30)
x = seq(0,100)
n = length(x)
e = rnorm(n, m=0, s=sqrt(x))
y = 3 + 2*x + e
plot(x,y, pch=21,bg="dodgerblue")

mod = lm(y~x)
bptest(mod)




##### ##### ##### ##### #####
### Section 3.3.3 (p98)
set.seed(30)

b0covered = numeric()
b1covered = numeric()

# heteroskedastic - trumpet
for(i in 1:1e4) {
  x = seq(0,10,0.5)
  n = length(x)
  e = rnorm(n, m=0, s=sqrt(x))
  y = 3 + 2*x + e
  mod = lm(y~x)
  ci = confint(mod)
  b0covered[i] = (3>ci[1,1]) && (3<ci[1,2])
  b1covered[i] = (2>ci[2,1]) && (2<ci[2,2])
}

mean(b0covered)
mean(b1covered)


# heteroskedastic - funnel
set.seed(30)
for(i in 1:1e4) {
  x = seq(0,10,0.5)
  n = length(x)
  e = rnorm(n, m=0, s=15-x)
  y = 3 + 2*x + e
  mod = lm(y~x)
  ci = confint(mod)
  b0covered[i] = (3>ci[1,1]) && (3<ci[1,2])
  b1covered[i] = (2>ci[2,1]) && (2<ci[2,2])
}

mean(b0covered)
mean(b1covered)


# heteroskedastic - bulge
set.seed(30)
for(i in 1:1e4) {
  x = seq(0,10,0.5)
  n = length(x)
  e = rnorm(n, m=0, s=11-2*abs(x-5))
  y = 3 + 2*x + e
  mod = lm(y~x)
  ci = confint(mod)
  b0covered[i] = (3>ci[1,1]) && (3<ci[1,2])
  b1covered[i] = (2>ci[2,1]) && (2<ci[2,2])
}

mean(b0covered)
mean(b1covered)



## prediction coverage (p102)
set.seed(30)

y1covered = numeric()
y2covered = numeric()
y3covered = numeric()

for(i in 1:1e4) {
  x = seq(0,10,0.5)
  n = length(x)
  e = rnorm(n, m=0, s=sqrt(x))
  y = 3 + 2*x + e
  mod = lm(y~x)
  ypr = predict(mod, newdata=data.frame(x=c(0,5,10)), interval="confidence")
  y1covered[i] = ( 3>ypr[1,2]) && ( 3<ypr[1,3])
  y2covered[i] = (13>ypr[2,2]) && (13<ypr[2,3])
  y3covered[i] = (23>ypr[3,2]) && (23<ypr[3,3])
}

mean(y1covered)
mean(y2covered)
mean(y3covered)





##### ##### ##### ##### #####
### Section 3.4 (p103)
#   Violent Crime extended example

rm(list=ls())


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")
library(lawstat)
library(lmtest)

dt = read.csv("http://rur.kvasaheim.com/data/crime.csv")
attach(dt)
summary(dt)

## Model
crimeMod = lm(vcrime00 ~ vcrime90)


## Residuals
e = residuals(crimeMod)
normoverlay(e)
shapiroTest(e)

# Normality of sample means
et = numeric()
for(i in 1:1e3) {
  x = sample(e, replace=TRUE)
  et[i] = sum( x )
}
shapiroTest(et)

# Constant Expected Value of Residuals
plot(vcrime90,e)
runs.test(e, order=vcrime90)

# Constant Variance of Residuals
plot(vcrime90,e)
bptest(crimeMod)

# Model results
summary(crimeMod)


## Graphic
par(mar=c(4,4,0,1)+0.5)
par(family="serif", las=1)
par(xaxs="i", yaxs="i")
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0,2500), ylim=c(0,2500) )

axis(1); axis(2)
title(xlab="Violent Crime Rate (1990)", line=2.75)
title(ylab="Violent Crime Rate (2000)", line=3.25)

xx = seq(0,2500)
yy = predict(crimeMod, newdata=data.frame(vcrime90=xx))

lines(xx,yy, col="steelblue", lwd=2)
points(vcrime90,vcrime00, pch=21, bg=1+(vcrime00>vcrime90))

# confidence interval for the parameters
confint(crimeMod)

# confidence interval for Y
predict(crimeMod, newdata=data.frame(vcrime90=100), interval="confidence")

# prediction interval for Y
predict(crimeMod, newdata=data.frame(vcrime90=100), interval="prediction")





##### ##### ##### ##### #####
### Section 3.5 (p108)
#   Violent Crime + extended example

rm(list=ls())

### Preamble
source("http://rfs.kvasaheim.com/rfs.R")
library(lawstat)
library(lmtest)

dt = read.csv("http://rur.kvasaheim.com/data/crime.csv")
attach(dt)
summary(dt)


## Model 1 (interaction)
modEd1 = lm(vcrime00 ~ gspcap90 * census9)
summary.aov(modEd1)

## Model 2 (additive)
modEd2 = lm(vcrime00 ~ gspcap90 + census9)

e = residuals(modEd2)
# Normality checking
normoverlay(e)
shapiroTest(e)

# Expected Value
plot(gspcap90, e)
runs.test(e, order=gspcap90)

plot(census9,e)
summary(aov(e~census9))

# Heteroskedasticity
plot(gspcap90,e)
hetero.test(e,gspcap90)

plot(census9,e)
fligner.test(e~census9)

bptest(modEd2)

# Model results
summary.aov(modEd2)
summary(modEd2)

# Mountain region as base
census9 = set.base(census9, "Mountain")
modEd3 = lm(vcrime00 ~ gspcap90 + census9)
summary(modEd3)

# Pacific region as base
census9 = set.base(census9, "Pacific")
modEd3 = lm(vcrime00 ~ gspcap90 + census9)
summary(modEd3)

# South Atlantic region as base
census9 = set.base(census9, "South Atlantic")
modEd3 = lm(vcrime00 ~ gspcap90 + census9)
summary(modEd3)

## Graphic
par(mar=c(4,4,0,1)+0.5, family="serif", las=1)
par(xaxs="i", yaxs="i")
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0,75), ylim=c(0,2500) )

axis(1); axis(2)
title(xlab="GSP per Capita (1990) [$000]", line=2.75)
title(ylab="Violent Crime Rate (2000)", line=3.5)

xx = seq(15,70)*1000
yyPac = predict(modEd2, newdata=data.frame(gspcap90=xx, census9="Pacific"))
yyMtn = predict(modEd2, newdata=data.frame(gspcap90=xx, census9="Mountain"))
yySAt = predict(modEd2, newdata=data.frame(gspcap90=xx, census9="South Atlantic"))

lines(xx/1000,yyPac, col="steelblue", lwd=2)
lines(xx/1000,yyMtn, col="gold", lwd=2)
lines(xx/1000,yySAt, col="pink", lwd=2)

points(gspcap90/1000,vcrime00, pch=21, bg="lightsteelblue")

# confidence interval for parameters
confint(modEd2)

# confidence interval for Y
predict(modEd2, newdata=data.frame(gspcap90=50000, census9="Pacific"), interval="confidence")

# prediction interval for Y
predict(modEd2, newdata=data.frame(gspcap90=50000, census9="Pacific"), interval="prediction")






##### ##### ##### ##### #####
### Section 3.6 (p116)
#   Cow extended example

rm(list=ls())

### Preamble
source("http://rfs.kvasaheim.com/rfs.R")
library(lawstat)
library(lmtest)

cows = read.csv("http://rur.kvasaheim.com/data/cows.csv")
summary(cows)
attach(cows)

# Model 1
mod1 = lm(propWin ~ yearPassed * chickens * religPct)
summary(mod1)


# Model 2
mod2 = lm(propWin ~ yearPassed * chickens * religPct - yearPassed:chickens:religPct)
mod2 = lm(propWin ~ (yearPassed + chickens + religPct)^2)
summary.aov(mod2)

# Model 3
mod3 = lm(propWin ~ yearPassed + chickens + religPct)
summary.aov(mod3)

# Multicollinearity
vif1 = lm(yearPassed ~ chickens + religPct)
vif2 = lm(chickens ~ yearPassed + religPct)
vif3 = lm(religPct ~ yearPassed + chickens)
1/(1-summary(vif1)$r.squared)
1/(1-summary(vif2)$r.squared)
1/(1-summary(vif3)$r.squared)

# OR
library(car)
vif(mod3)


# Predictions
DECIN = data.frame(yearPassed=9, chickens=0, religPct=48)
predict(mod3, newdata=DECIN)

# OR
predict(mod3, newdata=data.frame(yearPassed=9, chickens=0,religPct=48))



## Answering the question (p126)
# Initialize variables
outcome = numeric()
trials  = 1e6

# Coefficient estimates
b.intc =  0.151221
b.year = -0.020095
b.cban = -0.037331
b.rpct =  0.009452

# Coefficient standard errors
s.intc = 0.065938
s.year = 0.003577
s.cban = 0.019988
s.rpct = 0.001074

# Distributions (the "loop")
e.intc = rnorm(trials, m=b.intc, s=s.intc)
e.year = rnorm(trials, m=b.year, s=s.year)
e.cban = rnorm(trials, m=b.cban, s=s.cban)
e.rpct = rnorm(trials, m=b.rpct, s=s.rpct)
outcome = e.intc + e.year*9 + e.cban*0 + e.rpct*48

hist(outcome, main="", xlab="Proportion Vote for Ballot Measure", breaks=-1:99/100)
hist(outcome[outcome>0.50], main="", yaxt="n", breaks=-1:99/100, col=2, add=TRUE)
axis(1, at=0.50, labels="50%")

# the problem:
mean(outcome>1) + mean(outcome<0)








##### ##### ##### ##### #####
### Figure 3.9 (p129)
#   uses Section 3.6

par(las=1, family="serif")
par(xaxs="i", yaxs="i")
par(mar=c(3,3,0,0)+1)
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(-3,9), ylim=c(0,1) )

axis(1, at=seq(-3,9,2), label=seq(1997,2009,2))
axis(2, at=0:5/5)

title(xlab="Year of Referendum", line=2.5)
title(ylab="Support Proportion", line=3.0)



clip(-4,10, -1,2)
points(yearPassed,propWin, pch=21, bg="dodgerblue")

xx =  seq(-2,8)
yy0 = predict(mod3, newdata=data.frame(chickens=0, yearPassed=xx, religPct=mean(religPct)))
yy1 = predict(mod3, newdata=data.frame(chickens=1, yearPassed=xx, religPct=mean(religPct)))

lines(xx,yy0, col="honeydew4", lwd=2)
lines(xx,yy1, col="tomato3",   lwd=2)

legend( "bottomright",lwd=2, bty="n", col=c("tomato3","honeydew4"),
        legend=c("Ban on Chickens","No Ban on Chickens")
      )


## Second pane

par(las=1, family="serif")
par(xaxs="i", yaxs="i")
par(mar=c(3,3,0,0)+1)
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0,100), ylim=c(0,1) )

axis(1, at=seq(0,100,20) )
axis(2, at=0:5/5)

title(xlab="Percent Religious", line=2.5)
title(ylab="Support Proportion", line=3.0)



clip(-1,200, -1,2)
points(religPct,propWin, pch=21, bg="dodgerblue")

xx =  seq(min(religPct), max(religPct), length=4)
yy0 = predict(mod3, newdata=data.frame(chickens =0, yearPassed=8, religPct=xx))
yy1 = predict(mod3, newdata=data.frame(chickens =1, yearPassed=8, religPct=xx))

lines(xx,yy0, col="honeydew4", lwd=2)
lines(xx,yy1, col="tomato3",   lwd=2)

legend( "bottomright",lwd=2, bty="n", col=c("tomato3","honeydew4"),
        legend=c("Ban on Chickens","No Ban on Chickens")
      )










##### ##### ##### ##### #####
### Figure 3.10 (p130)
#   uses Section 3.6

par(las=1, family="serif")
par(xaxs="i", yaxs="i")
par(mar=c(3,0,0,0)+1)
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0,1), ylim=c(0,4.5) )

axis(1, at=0:4/4 )

title(xlab="Simulated Vote Results", line=2.5)


breaks = seq(-1,2,0.01)

tt = hist(outcome,breaks=breaks, plot=FALSE)

for(i in 1:length(breaks)) {
  ito="tan1"
  if(tt$breaks[i]<0.5) { ito="honeydew2" }
  rect( tt$breaks[i],0, tt$breaks[i+1],tt$density[i], col=ito)
}







#####
##### ##### ##### ##### #####
#####
