##### The Red Book (v0.704442)
##### Scripts for Chapter 4
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")





##### ##### ##### ##### #####
### Example 4.1 (p144)
#   uses Example 3.6


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")
library(lawstat)
library(lmtest)

cows = read.csv("http://rur.kvasaheim.com/data/cows.csv")
summary(cows)
attach(cows)

# Model
logitWin = logit(propWin)
modLgt = lm(logitWin ~ yearPassed + chickens + religPct)

summary(modLgt)

DECIN = data.frame(yearPassed=9, chickens=0, religPct=48)
voteLgt = predict(modLgt, newdata=DECIN)
voteEst = logistic(voteLgt)
voteEst



## Monte Carlo (p 146)

# Initialize variables
outcome = numeric()
trials  = 1e6

# Let's script the results for greater precision
modelSummary = summary(modLgt)
results = modelSummary$coefficients

# Coefficient estimates
b.intc = results[1,1]
b.year = results[2,1]
b.cban = results[3,1]
b.rpct = results[4,1]

# Coefficient standard errors
s.intc = results[1,2]
s.year = results[2,2]
s.cban = results[3,2]
s.rpct = results[4,2]

# Distributions (the "loop")
e.intc = rnorm(trials, m=b.intc, s=s.intc)
e.year = rnorm(trials, m=b.year, s=s.year)
e.cban = rnorm(trials, m=b.cban, s=s.cban)
e.rpct = rnorm(trials, m=b.rpct, s=s.rpct)
outcomeLgt = e.intc + e.year*9 + e.cban*0 + e.rpct*48

outcome = logit.inv(outcomeLgt)


# No problem:
mean(outcome>1) + mean(outcome<0)

# Odds
mean(outcome>0.5) / mean(outcome<0.5)

# Probability
mean(outcome>0.5)

# Prediction interval
quantile(outcome, c(0.025,0.975))





##### ##### ##### ##### #####
### Figure 4.3 (p147)
#   requires Example 4.1

hist(outcome, main="", xlab="Proportion Vote for Ballot Measure", breaks=-1:99/100)
hist(outcome[outcome>0.50], main="", yaxt="n", breaks=-1:99/100, col=2, add=TRUE)
axis(1, at=0.50, labels="50%")





##### ##### ##### ##### #####
### Example 4.2 (p148)

# Clean and Preamble 
rm(cows)
detach(cows)

gdp = read.csv("http://rur.kvasaheim.com/data/gdpcap.csv")
summary(gdp)
attach(gdp)

RUR = data.frame(hig=5.1, democracy=-7)
PNG = data.frame(hig=2.1, democracy=10)

# Naive model
mod = lm(gdpcap ~ hig + democracy)
predict(mod, newdata = PNG)

# Transformed model
modLog = lm(log(gdpcap) ~ democracy + hig)
summary(modLog)

estLog = predict(modLog, newdata=RUR)
exp(estLog)


# MC Estimation
b.int = 6.933298
b.dem = -0.002776
b.hig = 0.470225

s.int = 0.147873
s.dem = 0.011253
s.hig = 0.035855

e.int = rnorm(trials, m=b.int, s=s.int)
e.dem = rnorm(trials, m=b.dem, s=s.dem)
e.hig = rnorm(trials, m=b.hig, s=s.hig)

outcomeLog = e.int + e.dem*-7 + e.hig*5.1
est = exp(outcomeLog)

quantile(est, c(0.025,0.975))





##### ##### ##### ##### #####
### Figure 4.4 (p152)
#   requires Example 4.2


# Parameters
ito = c("tan1","honeydew2")
Q = quantile(est, c(0.025,0.5,0.975))
brkpts = seq(0,40000,500)
tt = hist(est, breaks=brkpts, plot=FALSE)

par(mar=c(3,1,1,1))
par(cex.lab=0.7, cex.axis=0.7)
par(xaxs="i",yaxs="i")
par(family="serif")


plot.new()
plot.window(xlim=c(0,40000), ylim=c(0,max(tt$density)))

for(i in 1:length(tt$density) ) {
  rr = ito[2-(tt$breaks[i]<Q[1] || tt$breaks[i]>Q[3])]
  rect(tt$breaks[i],0, tt$breaks[i+1],tt$density[i], col=rr)
}

axis(1, at=0:8*5000, label=c(0,5000,"10,000","15,000","20,000","25,000","30,000","35,000","40,000"))
axis(1, at=Q[2], label="")
mtext(side=1, at=Q[2], text=expression(tilde(x)), line=0.5)





##### ##### ##### ##### #####
### Prediction Interval (p153)
#   requires Figure 4.4

b.int =  6.933298
b.dem = -0.002776
b.hig =  0.470225
b.err =  0

s.int = 0.147873
s.dem = 0.011253
s.hig = 0.035855
s.err = 0.8841

e.int = rnorm(trials, m=b.int, s=s.int)
e.dem = rnorm(trials, m=b.dem, s=s.dem)
e.hig = rnorm(trials, m=b.hig, s=s.hig)
e.err = rnorm(trials, m=b.err, s=s.err)

outcomeLog = e.int + e.dem*-7 + e.hig*5.1 + e.err
est = exp(outcomeLog)

quantile(est, c(0.025,0.975))





##### ##### ##### ##### #####
### Section 4.2: South Sudanese Referendum

## Clean Up
detach(gdp)
rm(list=ls())

## Preamble
source("http://rfs.kvasaheim.com/rfs.R")
library(lawstat)
library(lmtest)

xsd = read.csv("http://rur.kvasaheim.com/data/xsd2011referendum.csv")
summary(xsd)
attach(xsd)

# Define variables
xsd$Valid = xsd$Votes-xsd$Invalid
pInv = xsd$Invalid/xsd$Votes
pSec = xsd$Secession/xsd$Valid


## Figure 4.5
par(xaxs="i",yaxs="i")
par(las=1, family="serif")
par(mar=c(4,4,0,0)+0.5)
par(cex.lab=1.2, cex.axis=0.9, font.lab=2)

plot.new()
plot.window( xlim=c(0,1), ylim=c(0,0.10) )

axis(1, at=0:10/10); axis(2)
title(xlab="Support for Secession",line=2.75)
title(ylab="Invalidation Rate", line=3.25)

clip(-1,2, -1,2)
points(pSec,pInv, pch=21, bg="thistle3")


## Logit-Transformed Model (without zeroes)
dr = which(xsd$Invalid==0)

p.ind <- xsd$Secession[-dr]/xsd$Votes[-dr]
p.inv <- xsd$Invalid[-dr]/xsd$Votes[-dr]
l.inv <- logit(p.inv)

model.xsd = lm(l.inv ~ p.ind)


## Figure 4.6
newX <- seq(0,1, length=1e4)
l.pred <- predict(model.xsd, newdata=data.frame(p.ind=newX), se.fit=TRUE)
p.pred <- logistic(l.pred$fit)

# Working-Hotelling
n = length(p.ind)
W = sqrt( 2 * qf(1-0.05, 2, n-2))
lcb.l = l.pred$fit - W * l.pred$se.fit
ucb.l = l.pred$fit + W * l.pred$se.fit

lcb = logistic(lcb.l)
ucb = logistic(ucb.l)

# Graphic
par(xaxs="i",yaxs="i")
par(las=1, family="serif")
par(mar=c(4,4,0,0)+0.5)
par(cex.lab=1.2, cex.axis=0.9, font.lab=2)

plot.new()
plot.window( xlim=c(0,1), ylim=c(0,0.10) )

axis(1, at=0:10/10); axis(2)
title(xlab="Support for Secession",line=2.75)
title(ylab="Invalidation Rate", line=3.25)

lines(newX, logistic(lcb.l), col="grey")
lines(newX, logistic(ucb.l), col="grey")
lines(newX, p.pred)

clip(-1,2, -1,2)
points(pSec,pInv, pch=21, bg="thistle3")





##### ##### ##### ##### #####
### Example 4.7 (p163)
#   requires Section 4.2

summaryHCE(model.xsd)






#####
##### ##### ##### ##### #####
#####
