##### The Red Book (v0.704442)
##### Scripts for Chapter 5
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")





##### ##### ##### ##### #####
### Example 5.1 (p185)

# Data
unemp = c(11.35, 11.41, 11.12, 11.08, 10.93, 10.86, 10.96,
11.05, 11.10, 10.87, 10.79, 10.76, 10.94, 10.94, 10.92,
11.01, 11.04, 11.16, 11.13, 11.14)
year = 1:20

# Model
mod = lm(unemp~year)

# Autocorrelation
E = residuals(mod)
autocor.test(E)

# GLS AR-1 Model
Sigma = diag(20)
Sigma = 0.5^abs(row(Sigma)-col(Sigma))

Y = matrix(year)
X = matrix( cbind(rep(1,20),unemp), ncol=2 )

# Coefficient estimates
solve(t(X)%*%solve(Sigma)%*%X) %*% t(X)%*%solve(Sigma)%*%Y




##### ##### ##### ##### #####
### Section 5.4 (p190)

# Data
barns  = 1:8
energy = c(848.9,476.9,350.9,289.2,251.7,225.8,209.7,193.9)
stdev  = c(7.8,9.2,9.4,10.2,7.4,9.3,7.2,5.3)
barns1 = 1/barns

# OLS
modOLS = lm(energy~barns1)
summary(modOLS)
confint(modOLS)

# WLS
modWLS = lm(energy~barns1, weight=stdev)
summary(modWLS)
confint(modWLS)




##### ##### ##### ##### #####
### Section 5.5 (p193): Elections in Ruritania

## Clean
rm(list=ls())

## Preamble
votes = read.csv("http://rur.kvasaheim.com/data/rur2013parl.csv")
attach(votes)

Valid = Total-Invalid
pPR = PR/Valid
pInv = Invalid/Total

# OLS
modOLS = lm(pInv~pPR)
summary(modOLS)

# WLS
modWLS = lm(pInv~pPR, weights=sqrt(Total))
summary(modWLS)


## Figure 5.2 (p196)
par(mar=c(4,4,0,0.5)+0.75)
par(xaxs="i", yaxs="i")
par(las=1, family="serif")
par( cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0.65,0.75), ylim=c(0,0.2) )

axis(1, at=seq(0.65,0.75,0.025)); axis(2, at=seq(0,0.2,0.02))
title(xlab="Support for PR", line=2.7)
title( ylab="Invalidation Rate", line=3.25)

xx = seq(min(pPR),max(pPR), length=1e4)
yO = predict(modOLS, newdata=data.frame(pPR=xx))
yW = predict(modWLS, newdata=data.frame(pPR=xx))

lines(xx,yW, col="red2", lwd=2)
lines(xx,yO, col="gold4", lwd=2)

points(pPR,pInv, pch=21, bg="tomato")

legend( "topright", bty="n",col=c("gold4","red2"), lwd=2,
	legend=c("OLS Model","WLS Model"))







#####
##### ##### ##### ##### #####
#####
