##### The Red Book (v0.704442)
##### Scripts for Chapter 6
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")

getBasicPlot = function(x,y) {

  par( mar=c(3,3,0,0)+0.5 )
  par(family="serif", las=1)
  par(xaxs="i", yaxs="i")
  par(cex=0.6)
  par(font.lab=2, cex.lab=1.2)
  par(lend=1)
  
  plot.new()
  plot.window(xlim=c(0,10), ylim=c(0,10))

  axis(1, at=0:10); axis(2, at=0:10)
  title(xlab="x (Independent Variable)", line=2.25)
  title(ylab="y (Dependent Variable)", line=2.5)

}





##### ##### ##### ##### #####
### Example 6.1 (p208)

## Preamble
library(quantreg)

dt = read.csv("http://rfs.kvasaheim.com/data/crime.csv")
summary(dt)
attach(dt)

# Median Model
mod1 = rq(vcrime00~vcrime90)
summary(mod1)





##### ##### ##### ##### #####
### Example 6.2 (p209)

mod3 = rq(pcrime00~pcrime90)
summary(mod3)






##### ##### ##### ##### #####
### Figure 6.1 (p210)

# OLS
mod4 = lm(pcrime00~pcrime90)

par(family="serif", las=1)
par(xaxs="i",yaxs="i")
par(mar=c(4,4,0,0)+0.75)
par(cex.lab=1.1, font.lab=2)

plot.new()
plot.window( xlim=c(0,9000), ylim=c(0,9000) )

axis(1); axis(2)
title(xlab="Property Crime Rate (1990)", line=2.50)
title(ylab="Property Crime Rate (2000)", line=3.00)

xx = seq(0,9000)
yM = predict(mod3, newdata=data.frame(pcrime90=xx))
yO = predict(mod4, newdata=data.frame(pcrime90=xx))

lines(xx,yO, col="gold")
lines(xx,yM, col="red")

points(pcrime90,pcrime00, pch=21, bg="dodgerblue")







##### ##### ##### ##### #####
### Example 6.3 (p212)

mod5 = rq(vcrime00~vcrime90, tau=0.10)
summary(mod5)






##### ##### ##### ##### #####
### Example 6.4 (p213)

mod6 = rq(pcrime00~pcrime90, tau=0.90)
summary(mod6)







##### ##### ##### ##### #####
### Figure 6.2 (p214)

par(family="serif", las=1)
par(xaxs="i",yaxs="i")
par(mar=c(4,4,0,0)+0.75)
par(cex.lab=1.1, font.lab=2)

plot.new()
plot.window( xlim=c(0,9000), ylim=c(0,9000) )

axis(1); axis(2)
title(xlab="Property Crime Rate (1990)", line=2.50)
title(ylab="Property Crime Rate (2000)", line=3.00)

xx = seq(0,9000)
for(i in 1:9) {
  quantile=i/10
  
  modXX = rq(pcrime00~pcrime90, tau=quantile)
  yy = predict(modXX, newdata=data.frame(pcrime90=xx))

  lines(xx,yy, col=grey(1-quantile))
}

points(pcrime90,pcrime00, pch=21, bg="dodgerblue")






##### ##### ##### ##### #####
### Example 6.5 (p215)

mod10 = rq(pcrime00~gspcap90)
summary(mod10)

mod10a = lm(pcrime00~gspcap90)
summary(mod10a)



##### ##### ##### ##### #####
### Figure 6.3 (p216)

par(family="serif", las=1)
par(xaxs="i",yaxs="i")
par(mar=c(4,4,0,0)+0.75)
par(cex.lab=1.1, font.lab=2)

plot.new()
plot.window( xlim=c(0,50), ylim=c(0,6000) )

axis(1); axis(2)
title(xlab="GSP per capita (1990) [000s]", line=2.50)
title(ylab="Property Crime Rate (2000)", line=3.00)

xx = seq(0,90000)
for(i in 1:9) {
  quantile=i/10
  
  modXX = rq(pcrime00~gspcap90, tau=quantile)
  yy = predict(modXX, newdata=data.frame(gspcap90=xx))

  lines(xx/1000,yy, col=grey(1-quantile))
}

points(gspcap90/1000,pcrime00, pch=21, bg="violet")








#####
##### ##### ##### ##### #####
#####
