##### The Red Book (v0.704442)
##### Scripts for Chapter 9
#####


### Preamble
source("http://rfs.kvasaheim.com/rfs.R")





##### ##### ##### ##### #####
### Example 9.1 (p273)

income = c(20,30,30,25,27, 90,25,60,65,125)
age = c(25,30,21,35,28, 80,55,40,40,25)
insurance = c(0,0,0,0,0, 1,1,1,1,1)

BOB = data.frame(age=65, income=125)

# OLS Model
modOLS = lm(insurance ~ age + income) 
summary(modOLS)

# Prediction for BOB
predict(modOLS, newdata=BOB)


# Equal Variances???
var( residuals(model)[insurance==1] ) 
var( residuals(model)[insurance==0] ) 

var.test(residuals(model)~insurance)


## Figure 9.1
par(cex=0.8,cex.lab=0.8,cex.axis=0.8)
par(mar=c(3,4,1,1)+0.1)

plot(insurance,residuals(model), pch=16, las=1, 
  ylab="Model residuals", xlim=c(-0.5,1.5), ylim=c(-0.5,0.5),
  xlab="Purchased tornado insurance\n\n", xaxt="n")
axis(1,label=c("No","Yes"), at=c(0,1))






##### ##### ##### ##### #####
### Figure 9.2 (p277)
x = -50:50/10
y = 3*x+4
z = logit.inv(y)

# Graphics
par(fig=c(0,1,0.55,1))
par(cex=0.8,cex.lab=0.7,cex.axis=0.8)
par(mar=c(0.5,6,1,1)+0.1)

plot(x,y,type="l", xlim=c(-5,5), ylim=c(-15,25), las=1, xlab="Independent Variable\n",
  ylab="Linear Predictor Value \n(logit units)\n", yaxt="n", xaxt="n")
abline(h=logit(0.500), col=2,lty=2)
axis(2, label=c(-10,expression(logit(tau)),10,20),at=c(-10,0,10,20), las=1)


par(fig=c(0,1,0,0.55), new=TRUE)
par(cex=0.8,cex.lab=0.7,cex.axis=0.8)
par(mar=c(4,6,0.5,1)+0.1)

plot(x,z,type="l", xlim=c(-5,5), ylim=c(0,1), las=1, xlab="Independent Variable\n",
  ylab="Predicted Probability\n(Logistic of the Linear Predictor Value)\n", yaxt="n")
abline(h=0.500, col=2,lty=2)
axis(2, label=c("0.00",0.25,expression(tau),0.75,"1.00"),at=0:4/4, las=1)





##### ##### ##### ##### #####
### Figure 9.3 (p283)

x = -100:100/15
y.logt = logit.inv(x)
y.prob = pnorm(x)
y.cauc = cauchit.inv(x)


par(cex=0.8,cex.lab=0.8,cex.axis=0.8)
par(mar=c(3,3,1,1)+0.1)

plot(x,y.cauc,type="l", xaxt="n", yaxt="n", xlab="", ylab="", col="#666666", ylim=c(0,1))
lines(x,y.prob, col=4, lwd=1)
lines(x,y.logt, col=2, lwd=1)
axis(1, at=0, label=expression(eta==0))
axis(2, at=0.5, label=expression(mu==0.5), las=1)
legend("bottomright", c("Logit","Probit","Cauchit"), col=c(2,4,"#666666"), lwd=1, bty="n")





##### ##### ##### ##### #####
### Figure 9.4 (p284)

x = -100:100/15
y.clog = cloglog.inv(x)
y.logl = loglog.inv(x)
y.logt = logit.inv(x)

par(cex=0.8,cex.lab=0.8,cex.axis=0.8)
par(mar=c(3,3,1,1)+0.1)

plot(x,y.logt,type="l", xaxt="n", yaxt="n", xlab="", ylab="", col="#666666", ylim=c(0,1))
lines(x,y.clog, col=4, lwd=1)
lines(x,y.logl, col=2, lwd=1)
axis(1, at=0, label=expression(eta==0))
axis(2, at=0.5, label=expression(mu==0.5), las=1)
legend("bottomright", c("Logit","Loglog","Complementary Loglog"), col=c("#666666",2,4), lwd=1, bty="n")






##### ##### ##### ##### #####
### Example 9.2 (p285)

## Clean
rm(list=ls())

## Preamble
source("http://rfs.kvasaheim.com/rfs.R")

coin = read.csv("http://rfs.kvasaheim.com/data/coinflips.csv")
names(coin)
attach(coin)

## Model
m1 = glm(head~trial, family=binomial(link="logit"), data=coin)
summary(m1)

## P[Head | Coin1]
predict(m1, newdata=data.frame(trial=1))
logit.inv( predict(m1, newdata=data.frame(trial=1)) )








##### ##### ##### ##### #####
### Figure 9.5 (p287)


par(cex.lab=1.2, font.lab=1)
par(mar=c(4,3,0,4)+0.5)
par(family="serif")
par(xaxs="i",yaxs="i")
par(xpd=TRUE)

plot.new()
plot.window(xlim=c(0,100), ylim=c(0,1))

axis(2,at=0:1,label=c("Tail","Head"),las=1 )
axis(4,at=0:10/10, label=sprintf("%2.1f",seq(0,1,0.1)), las=1)
axis(1,at=0:10*10, las=1)
segments(0,0,100,0, col="white")

mtext( expression(bold(paste("Probability of a Head,  ", pi))), side=4, line=2.9, font=2)
mtext("Flip Result", side=2, line=2, font=2)
mtext("Trial number", side=1, line=2.5, font=2)

newx = -100:200
thr  =  66.4

points(coin, pch=21, bg=3)
pred = predict(m1, newdata=data.frame(trial=newx))

lines(newx,logistic(pred), col="dodgerblue",lwd=3)
lines(newx,logistic(pred), col="white",lwd=1)
segments(thr,0,thr,1, col="#666666")
segments(0,0.5,100,0.5, col="#666666")

errors = which(coin$head[which(coin$trial<thr)]==1)
points(errors,rep(1,length(errors)), bg=2, pch=21)
errors = which(coin$head[which(coin$trial>thr)]==0)
points(floor(thr)+errors,rep(0,length(errors)), bg=2, pch=21)

axis(4,at=0:10/10, label=sprintf("%2.1f",seq(0,1,0.1)), las=1)






##### ##### ##### ##### #####
### Section 9.5 (p288)

#   Accuracy Rate
accuracy(data=coin, y=coin$head, model=m1, t=0.500)

#   Maximum Accuracy
a = numeric()
for(i in 1:100) {
  t = i/100
  a[i] = accuracy(coin, coin$head, m1, t=t)
}
max(a)
which.max(a)


### Figure 9.6 (p291)

## Create data (30s to run)

B = 1e4
H = coin$head
cnum = coin$trial
acc = matrix(NA, ncol=100, nrow=B)

for( j in 1:B ) {
  thisSample = sample(100, replace=TRUE)
  mod = glm(H[thisSample]~cnum[thisSample], family=binomial)
  a = numeric()
  for(tau in 1:100) {
    a[tau] = accuracy( cnum, H, mod, t=tau/100 )
  }
  acc[j, ] = a
}


# Summary information for graphic

accMEAN = apply(acc,2,mean)
accUPPR = apply(acc,2,quantile,0.975)
accLOWR = apply(acc,2,quantile,0.025)

# actual values for graphic

mod = glm(H~cnum, family=binomial)

areal = numeric()
for(t in 1:100) {
  tau = t/100
  areal[t] = accuracy( coin, coin$head, mod, t=tau )
}


# Graphic

par(las=1)
par(family="serif")
par(mar=c(3,3,0,0)+0.75)
par(xaxs="i",yaxs="i")
par(cex.lab=1.2, font.lab=2)

plot.new()
plot.window( xlim=c(0,1), ylim=c(0.3,0.85) )

polygon( c(cnum/100,rev(cnum/100)), c(accUPPR,rev(accLOWR)), 
	border=FALSE, col=rgb(0,0.7,1,0.1) )

lines(cnum/100,accLOWR,col=rgb(0,0.7,1,0.3))
lines(cnum/100,accUPPR,col=rgb(0,0.7,1,0.3))

abline(v=0.500, col="black", lty=3)
abline(h=0.660, col="black", lty=3)



lines( cnum/100, areal)
clip(-1,2,-1,2)
points( cnum/100, areal, pch=21,bg="navy")

axis(1, at=0:10/10)
axis(2, at=0:10/10)
title(xlab="Threshold", line=2.2 )
title(ylab="Raw Accuracy", line=2.6)

points(0.48,areal[48],cex=2)







##### ##### ##### ##### #####
### Section 9.5.4 (p292)

# True-Positive Rate
TPR = function(data, y, model, t) {
  t = logit(t)
  pp = predict(model)
  if( t<min(pp) ) { return(1) }
  if( t>max(pp) ) { return(0) }
  tt = table(pp>t,y==1)
  tt[2,2]/(tt[2,2]+tt[1,2])
}

# False-Positive Rate
FPR <- function(data, y, model, t) {
  t <- logit(t)
  pp <- predict(model)
  if( t<min(pp) ) { return(1) }
  if( t>max(pp) ) { return(0) }
  tt <- table(pp>t,y==1)
  tt[2,1]/(tt[1,1]+tt[2,1])
}

# ROC curve
tpr = numeric()
fpr = numeric()

for(i in 1:1000) {
  tpr[i] = TPR(coin, coin$head, m1, t=i/1000)
  fpr[i] = FPR(coin, coin$head, m1, t=i/1000)
}



### Figure 9.7 (p293)
#   Simple ROC curve

par(mar=c(4,4,1,1)+0.1)
plot(fpr,tpr, type="l", lwd=2, las=1, xlab="False Positive Rate", ylab="True Positive Rate")
segments(0,0,1,1)


# AUC calculation
-sum(tpr*diff(c(1,fpr)))



### Figure 9.8 (p294)
#   ROC curve using Epi

library(Epi)

m1 = glm(head~trial, family=binomial(link=logit), data=coin)
summary(m1)
pred = logistic( predict(m1) )


# Figure 7.9: Epi ROC curve
ROC(test=pred, stat=coin$head, plot="ROC", las=1)










##### ##### ##### ##### #####
### Section 9.6 (p295)


### Figure 9.9: Logit and cloglog graph
par(mar=c(4,4,1,1)+0.1)

plot(-60:60/10,logistic(-60:60/10),type="l", las=1, col=1,
  xlab=expression(f(eta)), ylab=expression(mu) , lwd=2)
lines(-60:60/10,cloglog.inv(-60:60/10), col=4, lwd=2)
legend("topleft",c("Logit","Complementary Loglog"),lwd=2,col=c(1,4),bty="n")


### Table 9.4 (p297)
mc = glm(head~trial, family=binomial(link=cloglog), data=coin)
summary(mc)

# Prediction (p297)
pred = predict(mc, newdata=data.frame(trial=1))
cloglog.inv(pred)


### Figure 9.10 (p298)

par(mar=c(4,4,1,1)+0.1)

plot(-60:60/10,logistic(-60:60/10),type="l", las=1, col=1,
  xlab=expression(f(eta)), ylab=expression(mu) , lwd=2)
lines(-60:60/10,loglog.inv(-60:60/10), col=4, lwd=2)
legend("topleft",c("Logit","Loglog"),lwd=2,col=c(1,4),bty="n")


# Prediction (p297)
mg = glm(head~trial, family=binomial(link=make.link("loglog")), data=coin)
summary(mg)
pred = predict(mg, newdata=data.frame(trial=1))
loglog.inv(pred)








##### ##### ##### ##### #####
### Section 9.7 (p298)
#   Model Selection


# Akaike Information Criteria
AIC(m1)
AIC(mc)
AIC(mg)


# Bayesian Information Criteria
BIC(m1)
BIC(mc)
BIC(mg)


# Likelihood Ratio Test
logLik(m1)
logLik(mc)
logLik(mg)



### Example 9.3
pchisq(4,df=2, lower.tail=FALSE)













#####
##### ##### ##### ##### #####
#####
